package com.ruoyi.system.service.impl;

import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.system.domain.changeBalance.EChangeBalance;
import com.ruoyi.system.domain.changeBalance.EChangeBalanceQuery;
import com.ruoyi.system.mapper.EChangeBalanceMapper;
import com.ruoyi.system.service.IEChangeBalanceService;
import com.ruoyi.system.service.IEUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;

/**
 * 余额变动Service业务层处理
 *
 * @author qzz
 * @date 2024-09-28
 */
@Service
public class EChangeBalanceServiceImpl implements IEChangeBalanceService
{
    @Resource
    private EChangeBalanceMapper rlChangeBalanceMapper;

    @Resource
    private IEUserService userService;

    @Autowired
    private TransactionTemplate transactionTemplate;

    /**
     * 查询余额变动
     *
     * @param changeId 余额变动主键
     * @return 余额变动
     */
    @Override
    public EChangeBalance selectRlChangeBalanceByChangeId(Long changeId)
    {
        return rlChangeBalanceMapper.selectRlChangeBalanceByChangeId(changeId);
    }

    /**
     * 查询余额变动列表
     *
     * @param rlChangeBalance 余额变动
     * @return 余额变动
     */
    @Override
    @DataScope(merchantAlias = "u")
    public List<EChangeBalance> selectRlChangeBalanceList(EChangeBalance rlChangeBalance)
    {
        return rlChangeBalanceMapper.selectRlChangeBalanceList(rlChangeBalance);
    }

    /**
     * 根据用户id 查询余额变动列表
     *
     * @param RlUserId 用户id
     * @return 余额变动
     */
    @Override
    public List<EChangeBalance> selectRlChangeBalanceListByUserId(EChangeBalanceQuery query, Long RlUserId)
    {
        query.setOwnerId(RlUserId);
        return rlChangeBalanceMapper.selectRlChangeBalanceList(query);
    }

    /**
     * 新增余额变动
     *
     * @param rlChangeBalance 余额变动
     * @return 结果
     */
    @Override
    public int insertRlChangeBalance(EChangeBalance rlChangeBalance)
    {
        rlChangeBalance.setCreateTime(DateUtils.getNowDate());
        return rlChangeBalanceMapper.insertRlChangeBalance(rlChangeBalance);
    }

    /**
     * 修改余额变动
     *
     * @param rlChangeBalance 余额变动
     * @return 结果
     */
    @Override
    public int updateRlChangeBalance(EChangeBalance rlChangeBalance)
    {
        return rlChangeBalanceMapper.updateRlChangeBalance(rlChangeBalance);
    }

    /**
     * 批量删除余额变动
     *
     * @param changeIds 需要删除的余额变动主键
     * @return 结果
     */
    @Override
    public int deleteRlChangeBalanceByChangeIds(Long[] changeIds)
    {
        return rlChangeBalanceMapper.deleteRlChangeBalanceByChangeIds(changeIds);
    }

    /**
     * 删除余额变动信息
     *
     * @param changeId 余额变动主键
     * @return 结果
     */
    @Override
    public int deleteRlChangeBalanceByChangeId(Long changeId)
    {
        return rlChangeBalanceMapper.deleteRlChangeBalanceByChangeId(changeId);
    }

    /**
     * 根据订单信息生成账变记录
     *
     * @param orderNo 订单号
     * @param outTradeNo 第三方流水号
     * @param type 收支类型：1-收入；2-支出
     * @param busType 业务类型
     * @param payFee 支付金额
     * @param userId 用户id
     * @param userName 用户名称
     * @param phone 用户电话
     * @param userType 用户类型
     * @return 结果
     */
    @Override
    public int generateChanggeBalance(String orderNo,String outTradeNo,String type,String busType,BigDecimal payFee,Long userId,String userName,String phone,String userType) {
        EChangeBalance rlChangeBalance = new EChangeBalance();
        rlChangeBalance.setOrderNo(orderNo);
        rlChangeBalance.setOutTradeNo(outTradeNo);
        rlChangeBalance.setType(type);
        rlChangeBalance.setBusType(busType);
        BigDecimal balance = userService.selectUserBalanceById(userId);
        rlChangeBalance.setBeforeBalance(balance);
        if(type.equals("1")){
            rlChangeBalance.setAfterBalance(balance.add(payFee));
        }else{
            rlChangeBalance.setAfterBalance(balance.subtract(payFee));
        }
        rlChangeBalance.setAmount(payFee);
        rlChangeBalance.setOwnerId(userId);
        rlChangeBalance.setOwnerName(userName);
        rlChangeBalance.setOwnerType(userType);
        rlChangeBalance.setOwnerPhone(phone);
        rlChangeBalance.setCreateTime(DateUtils.getNowDate());
        // 根据 busType 生成不同的 reason 描述
        BigDecimal changeAmount  = rlChangeBalance.getAmount();
        String reason = "";
        switch (busType) {
            case "1":
                reason = "订单收入：" + orderNo;
                break;
            case "2":
                reason = "续单订单收入：" + orderNo;
                break;
            case "4":
                reason = "订单退款：" + orderNo;
                changeAmount = changeAmount.negate();
                break;
            case "5":
                reason = "提现";
                changeAmount = changeAmount.negate();
                break;
            case "6":
                reason = "提现失败";
                break;
            case "8":
                reason = "短信扣费";
                changeAmount = changeAmount.negate();
                break;
            default:
                reason = "未知业务类型";
                break;
        }
        rlChangeBalance.setReason(reason);
        BigDecimal finalChangeAmount = changeAmount;
        Boolean execute = transactionTemplate.execute(e -> {
            int i = insertRlChangeBalance(rlChangeBalance);
            if (i <= 0) {
                throw new RuntimeException("账变记录插入失败");
            }
            int i2 = userService.changeBalance(finalChangeAmount,userId);
            return Boolean.TRUE;
        });
        if(!execute)throw new ServiceException("账变记录插入失败");
        return 1;
    }

    /**
     *  最后账变记录
     */
    @Override
    public EChangeBalance lastChangeBalance(Long userId) {
        return rlChangeBalanceMapper.lastChangeBalance(userId);
    }
}
