package com.ruoyi.system.service.impl;

import java.math.BigDecimal;
import java.util.Comparator;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.map.GeoUtils;
import com.ruoyi.system.city.domain.RlCity;
import com.ruoyi.system.city.service.IRlCityService;
import com.ruoyi.system.device.service.DeviceService;
import com.ruoyi.system.domain.room.ERoomVO;
import com.ruoyi.system.service.IERoomService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.EStoreMapper;
import com.ruoyi.system.domain.store.EStore;
import com.ruoyi.system.domain.store.EStoreVO;
import com.ruoyi.system.domain.store.EStoreQuery;
import com.ruoyi.system.service.IEStoreService;

import javax.annotation.Resource;

/**
 * 店铺Service业务层处理
 *
 * @author qzz
 * @date 2024-11-20
 */
@Slf4j
@Service
public class EStoreServiceImpl implements IEStoreService
{
    @Autowired
    private EStoreMapper storeMapper;

    @Resource
    private IRlCityService cityService;

    @Resource
    public IERoomService roomService;

    @Autowired
    private DeviceService smDeviceService;

    /**
     * 查询店铺
     *
     * @param storeId 店铺主键
     * @return 店铺
     */
    @Override
    public EStoreVO selectEStoreByStoreId(Long storeId)
    {
        EStoreVO storeVO = storeMapper.selectEStoreByStoreId(storeId);
        List<ERoomVO> eRoomVOS = roomService.selectERoomListByStoreId(storeId);
        storeVO.setRoomList(eRoomVOS);
        return storeVO;
    }

    /**
     * 查询店铺列表
     *
     * @param eStore 店铺
     * @return 店铺
     */
    @Override
    public List<EStoreVO> selectEStoreList(EStoreQuery eStore)
    {
        return storeMapper.selectEStoreList(eStore);
    }

    /**
     * 新增店铺
     *
     * @param eStore 店铺
     * @return 结果
     */
    @Override
    public int insertEStore(EStore eStore)
    {
        eStore.setCreateTime(DateUtils.getNowDate());
        return storeMapper.insertEStore(eStore);
    }

    /**
     * 修改店铺
     *
     * @param eStore 店铺
     * @return 结果
     */
    @Override
    public int updateEStore(EStore eStore)
    {
        eStore.setUpdateTime(DateUtils.getNowDate());
        return storeMapper.updateEStore(eStore);
    }

    /**
     * 批量删除店铺
     *
     * @param storeIds 需要删除的店铺主键
     * @return 结果
     */
    @Override
    public int deleteEStoreByStoreIds(Long[] storeIds)
    {
        return storeMapper.deleteEStoreByStoreIds(storeIds);
    }

    /**
     * 删除店铺信息
     *
     * @param storeId 店铺主键
     * @return 结果
     */
    @Override
    public int deleteEStoreByStoreId(Long storeId)
    {
        return storeMapper.deleteEStoreByStoreId(storeId);
    }

    /**
     * 根据定位获取附近店铺列表
     */
    @Override
    public List<EStoreVO> getStoreListByLocation(EStoreQuery query) {
        // 根据定位查询出是哪个城市，获取该城市下的所有店铺
        RlCity city = cityService.getCityByLocation(query.getPhoneLon(), query.getPhoneLat());
        if(!Objects.isNull(city)){
            List<EStoreVO> eStoreVOS = storeMapper.selectEStoreByCityId(city.getCityId());
            for (EStoreVO eStoreVO : eStoreVOS) {
                eStoreVO.setDistance(getDistance(query.getPhoneLon(), query.getPhoneLat(), eStoreVO.getLng(), eStoreVO.getLat()));
            }
            return eStoreVOS;
        }
        return null;
    }

    /**
     * 开大门
     */
    @Override
    public boolean openGate(Long storeId) {
        EStoreVO eStoreVO = storeMapper.selectEStoreByStoreId(storeId);
        return  smDeviceService.switchDevice(eStoreVO.getGateId(), true);
    }

    /**
     * 关大门
     */
    @Override
    public boolean closeGate(Long storeId) {
        EStoreVO eStoreVO = storeMapper.selectEStoreByStoreId(storeId);
        return  smDeviceService.switchDevice(eStoreVO.getGateId(), false);
    }

    private Double getDistance(String phoneLon, String phoneLat, BigDecimal lng, BigDecimal lat) {
        double[] point1 = {Double.parseDouble(phoneLon), Double.parseDouble(phoneLat)};
        double[] point2 = {lng.doubleValue(), lat.doubleValue()};
        double distanceInMeters = GeoUtils.haversineDistance(point1, point2);
        return Math.round((distanceInMeters / 1000) * 10) / 10.0;
    }
}
